/*
	       File: FileCopy.cs
	Create Date: 2007-08-13
	    Version:

	Description:


	 Change log:


*/
using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections;
using System.Drawing;
using System.IO;
using System.Workflow.ComponentModel;
using System.Workflow.ComponentModel.Design;
using System.Workflow.ComponentModel.Compiler;
using System.Workflow.ComponentModel.Serialization;
using System.Workflow.Runtime;
using System.Workflow.Activities;
using System.Workflow.Activities.Rules;

using GuerillaProgrammer.Components;
using GuerillaProgrammer.Components.Logging;
using GuerillaProgrammer.FileProcessor.Settings;
using GuerillaProgrammer.FileProcessor;

namespace GuerillaProgrammer.FileProcessor.Activities
{
	public partial class FileCopy: Activity
    {
        #region Private member fields
        private bool _OverWriteFile;
        private string _SourceFilePath;
        private string _DestinationFilePath;
        #endregion

        #region Constructors and cleanup methods
        public FileCopy()
        {
            InitializeComponent();
        }
        #endregion

        #region Private implementation methods
        #endregion

        #region Class properties
        public static DependencyProperty SourceFilePathProperty = System.Workflow.ComponentModel.DependencyProperty.Register("SourceFilePath", typeof(string), typeof(FileCopy));
        [Description("Source for the workitem file.  If in file mode the path needs to be complete including a filename.\nIn directory mode the path will be have the workitem's filename appended to it.")]
        [Category("File Copy")]
        [Browsable(true)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        public string SourceFilePath
        {
            get { return _SourceFilePath; }
            set { _SourceFilePath = value; }
        }

        public static DependencyProperty DestinationFilePathProperty = System.Workflow.ComponentModel.DependencyProperty.Register("DestinationFilePath", typeof(string), typeof(FileCopy));
        [Description("Destination for the workitem file.  If in file mode the path needs to be complete including a filename.\nIn directory mode the path will be have the workitem's filename appended to it.")]
        [Category("File Copy")]
        [Browsable(true)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        public string DestinationFilePath
        {
            get { return _DestinationFilePath; }
            set { _DestinationFilePath = value; }
        }

        public static DependencyProperty OverWriteFileProperty = System.Workflow.ComponentModel.DependencyProperty.Register("OverWriteFile", typeof(bool), typeof(FileCopy));
        [Description("Over write the file if it exists")]
        [Category("File Copy")]
        [Browsable(true)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        public bool OverWriteFile
        {
            get { return _OverWriteFile; }
            set { _OverWriteFile = value; }
        }
        #endregion

        #region Events handlers
        protected override ActivityExecutionStatus Execute(ActivityExecutionContext executionContext)
        {
            Logger.LogEnter();

            IProcessorDataService dataService = executionContext.GetService(typeof(IProcessorDataService)) as IProcessorDataService;
            if (null == dataService)
            {
                throw new InvalidOperationException("Can't get the service!");
            }

            try
            {
                WorkItem workItem = dataService.GetWorkItem();
                StringProcessor stringProcessor = new StringProcessor(new FileProcessorSymbolTable(dataService.GetConfiguration(), workItem));
                string processedSource = stringProcessor.SubstituteStringValues(_SourceFilePath);
                string processedDestination = stringProcessor.SubstituteStringValues(_DestinationFilePath);

                Logger.LogInfo("FileCopy", "File copy mode.\nCopying ({0}) to ({1})\n", processedSource, processedDestination);
                File.Copy(processedSource, processedDestination, _OverWriteFile);
            }
            catch(Exception exc)
            {
                Logger.LogError("FileCopy", "An error occured while attempting to copy a file.\nError: {0}", exc.Message);
                throw exc;
            }
            Logger.LogLeave();
            return base.Execute(executionContext);
        }
        #endregion

        #region Public interface
        #endregion
	}
}
