//*******************************************************************************
// Applicaiton Logging Components
// 
//*******************************************************************************
// Copyright  GuerillaProgrammer.com.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//*******************************************************************************
//     File: Logger.cs
//  Created: 2007-33-10
//   Author: Jake Watkins
//
//  Description:
//
//  EntLib Logging
//
//
//*******************************************************************************


using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;

using Microsoft.Practices.EnterpriseLibrary.Logging;
using System.Reflection;

namespace GuerillaProgrammer.Components.Logging
{
    public sealed class Logger
    {
        #region Private member fields
        #endregion

        #region Constructors and cleanup methods
        private Logger()
        {
        }
        #endregion

        #region Private implementation methods
        private static void LogMessage(string method, int priority, string title, string message, params object[] values)
        {
            if (null != values)
            {
                message = string.Format(message, values);
            }

            LogMessage(method, priority, title, message);
        }

        private static void LogMessage(string method, int priority, string title, string message)
        {
            try
            {
                LogEntry entry = new LogEntry();
                string messageBody = string.Format("Method: ({0}) - {1}", method, message);
                entry.Title = title;
                entry.Priority = priority;
                entry.Message = messageBody;
                Microsoft.Practices.EnterpriseLibrary.Logging.Logger.Write(entry);
            }
            catch (Exception exc)
            {
                System.Diagnostics.Debug.Write("Error " + exc.Message);
            }
        }
        private static string GetCallingMethodName()
        {
            StackFrame sfFrame = new StackFrame(2);
            MethodBase mbMethod = sfFrame.GetMethod();
            Type tParentClass = mbMethod.DeclaringType;

            return string.Format("{0}.{1}", tParentClass.FullName, mbMethod.Name);
        }
        #endregion

        #region Class attributes
        #endregion

        #region Events and delegates
        #endregion

        #region Public interface
        public static void LogEnter()
        {
            LogInfo("ENTERING", "Entering {0}", GetCallingMethodName(), 0);
        }
        public static void LogLeave()
        {
            LogInfo("LEAVING", "Leaving {0}", GetCallingMethodName());
        }
        public static void LogInfo(string title, string message, params object[] values)
        {
            if (values != null)
                message = string.Format(message, values);

            LogMessage(GetCallingMethodName(), 0, title, message);
        }
        public static void LogWarning(string title, string message, params object[] values)
        {
            if (values != null)
                message = string.Format(message, values);

            LogMessage(GetCallingMethodName(), 1, title, message);
        }
        public static void LogError(string title, string message, params object[] values)
        {
            if (values != null)
                message = string.Format(message, values);

            LogMessage(GetCallingMethodName(), 2, title, message);
        }
        #endregion
    }
}
